/*
	IC Types.h
	
	Header file for the types used in the IC system, also defines some constants and enumerations used throughout
	the IC system.
	
	This code is mostly based upon the work done by the IC pascal team.
*/

#ifndef __H_IC_Types__
#define __H_IC_Types__

#ifndef __ALIASES__
#include <Aliases.h>
#endif

#include <Components.h>

// ICError Codes
enum {
	icPrefNotFoundErr		= -666,		/* preference not found (duh!) */
	icPermErr				= -667,		/* cannot set preference */
	icPrefDataErr			= -668,		/* problem with preference data */
	icInternalErr			= -669,		/* hmm, this is not good */
	icTruncatedErr			= -670,		/* more data was present than was returned */
	icNoMoreWritersErr		= -671,		/* you cannot begin a write session because someone else is already doing it */
	icNothingToOverrideErr	= -672,		/* no component for the override component to capture */
	icNoURLErr			= -673,		/* no URL found */
	icConfigNotFoundErr		= -674,		/* no configuration was found */
	icConfigInappropriateErr	= -675		/* incorrect manufacturer code */
	
	// Codes added by dnebing@epix.net
	,icNotEnabled			= -676		/* The compoent and link in glue were disabled in the switch glue */
};

// IC Attributes - Attribute settings for the preferences
enum {
	ICattr_no_change			= 0xFFFFFFFFL,		/* supply this to ICSetPref to tell it not to change the attributes */
	ICattr_locked_bit			= 0,				/* bits in the preference attributes */
	ICattr_locked_mask			= 0x00000001L,	/* masks for the above */
	ICattr_volatile_bit			= 1,
	ICattr_volatile_mask		= 0x00000002L,
	icNoUserInteraction_bit		= 0,
	icNoUserInteraction_mask		= 0x00000001L
};

// ICPerm Values - Permissions for access to the preference values
enum {
	icNoPerm = 0,
	icReadOnlyPerm = 1,
	icReadWritePerm = 2
};

// OSTypes for the prefs files
#define kICFileType			'ICAp'
#define kICCreator			'ICAp'

#define ICfiletype kICFileType
#define ICcreator kICCreator

// Default file name to use - Can be overridden by the applications
#define kICDefaultFileName	"\pInternet Preferences"

// Constant values for the component and version information
#define internetConfigurationComponentType		'PREF'		/* the component type */
#define internetConfigurationComponentSubType	'ICAp'		/* the component subtype */

#define internetConfigurationComponentInterfaceVersion0	0x00000000L
#define internetConfigurationComponentInterfaceVersion1	0x00010000L
#define internetConfigurationComponentInterfaceVersion2	0x00020000L
#define internetConfigurationComponentInterfaceVersion21	0x00020001L

/* current version number is version 2.1 */

#define internetConfigurationComponentInterfaceVersion internetConfigurationComponentInterfaceVersion2

typedef ComponentInstance internetConfigurationComponent;

// ICDirSpec - Structure for specifying a directory
#if defined(powerc) || defined (__powerc)
#pragma options align=mac68k
#endif
struct ICDirSpec {
	short vRefNum;
	long dirID;
};
#if defined(powerc) || defined(__powerc)
#pragma options align=reset
#endif

typedef struct ICDirSpec ICDirSpec;

typedef ICDirSpec ICDirSpecArray[4];			/* an array of the above */
typedef ICDirSpec* ICDirSpecArrayPtr;			/* a pointer to that array */

typedef long ICAttr;						/* type for preference attributes */
typedef long ICError;						/* type for error codes */

typedef Ptr ICInstance;						/* opaque type for preference reference */

typedef unsigned char ICPerm;

#if defined(powerc) || defined (__powerc)
#pragma options align=mac68k
#endif
struct ICConfigRef {
	OSType manufacturer;
	/* other private data follows */
};
#if defined(powerc) || defined(__powerc)
#pragma options align=reset
#endif

typedef struct ICConfigRef ICConfigRef, *ICConfigRefPtr, **ICConfigRefHandle;

/* ICFontRecord - Font record */
#if defined(powerc) || defined (__powerc)
#pragma options align=mac68k
#endif
struct ICFontRecord {	
	short size;
	Style face;
	char pad;
	Str255 font;
};
#if defined(powerc) || defined(__powerc)
#pragma options align=reset
#endif
typedef struct ICFontRecord ICFontRecord, *ICFontRecordPtr, **ICFontRecordHandle;

/* ICCharTable - Character translation table */
#if defined(powerc) || defined (__powerc)
#pragma options align=mac68k
#endif
struct ICCharTable {
	unsigned char net_to_mac[256];
	unsigned char mac_to_net[256];
};
#if defined(powerc) || defined(__powerc)
#pragma options align=reset
#endif
typedef struct ICCharTable ICCharTable, *ICCharTablePtr, **ICCharTableHandle;

/* ICAppSpec - */
#if defined(powerc) || defined (__powerc)
#pragma options align=mac68k
#endif
struct ICAppSpec {
	OSType fCreator;
	Str63 name;
};
#if defined(powerc) || defined(__powerc)
#pragma options align=reset
#endif
typedef struct ICAppSpec ICAppSpec, *ICAppSpecPtr, **ICAppSpecHandle;

/* ICFileInfo - */
#if defined(powerc) || defined (__powerc)
#pragma options align=mac68k
#endif
struct ICFileInfo {
	OSType fType;
	OSType fCreator;
	Str63 name;
};
#if defined(powerc) || defined(__powerc)
#pragma options align=reset
#endif
typedef struct ICFileInfo ICFileInfo, *ICFileInfoPtr, **ICFileInfoHandle;

/* ICFileInfo - */
#if defined(powerc) || defined (__powerc)
#pragma options align=mac68k
#endif
struct ICFileSpec {
	Str31 vol_name;
	long vol_creation_date;
	FSSpec fss;
	AliasRecord alias;
	/* plus extra data, aliasSize 0 means no alias manager present when
			ICFileSpecification was created */
};
#if defined(powerc) || defined(__powerc)
#pragma options align=reset
#endif
typedef struct ICFileSpec ICFileSpec, *ICFileSpecPtr, **ICFileSpecHandle;

enum {
	ICfile_spec_header_size = sizeof(ICFileSpec) - sizeof(AliasRecord)
};

/* ICMapEntry - Structure for defining how a file maps to an application */
#if defined(powerc) || defined (__powerc)
#pragma options align=mac68k
#endif
struct ICMapEntry {
	short total_length;
	short fixed_length;
	short version;
	OSType file_type;
	OSType file_creator;
	OSType post_creator;
	long flags;
	/* variable part starts here */
	Str255 extension;
	Str255 creator_app_name;
	Str255 post_app_name;
	Str255 MIME_type;
	Str255 entry_name;
};
#if defined(powerc) || defined(__powerc)
#pragma options align=reset
#endif
typedef struct ICMapEntry ICMapEntry, *ICMapEntryPtr, **ICMapEntryHandle;

enum {
	ICmap_binary_bit = 0,						/* file should be transfered in binary as opposed to text mode */
	ICmap_binary_mask = 0x00000001,
	ICmap_resource_fork_bit = 1,					/* the resource fork of the file is significant */
	ICmap_resource_fork_mask = 0x00000002,
	ICmap_data_fork_bit = 2,						/* the data fork of the file is significant */
	ICmap_data_fork_mask = 0x00000004,

	ICmap_post_bit = 3,							/* post process using post fields */
	ICmap_post_mask = 0x00000008,

	ICmap_not_incoming_bit = 4,					/* ignore this mapping for incoming files */
	ICmap_not_incoming_mask = 0x00000010,
	ICmap_not_outgoing_bit = 5,					/* ignore this mapping for outgoing files */
	ICmap_not_outgoing_mask = 0x00000020,

	ICmap_fixed_length = 22						/* number in fixed_length field */
};

/* ICServiceEntry - specific name to port binding for a given service */
#if defined(powerc) || defined (__powerc)
#pragma options align=mac68k
#endif
struct ICServiceEntry {
	Str255 name;
	short port;
	short flags;
};
#if defined(powerc) || defined(__powerc)
#pragma options align=reset
#endif
typedef struct ICServiceEntry ICServiceEntry, *ICServiceEntryPtr, **ICServiceEntryHandle;

/* ICServices - IC Equivalent of the /etc/services file */
#if defined(powerc) || defined (__powerc)
#pragma options align=mac68k
#endif
struct ICServices {
	short count;
	ICServiceEntry services[1];
};
#if defined(powerc) || defined(__powerc)
#pragma options align=reset
#endif
typedef struct ICServices ICServices, *ICServicesPtr, **ICServicesHandle;

enum {
	ICservices_tcp_bit = 0,
	ICservices_tcp_mask = 0x00000001,
	ICservices_udp_bit = 1,
	ICservices_udp_mask = 0x00000002
	/* both bits can be set, which means the service is both TCP and UDP, eg daytime */
};

#endif /* __H_IC_Types__ */








