/*
	syslog component.h
	
	Header file describing the API for the component interface.
*/

#pragma once

#ifndef __H_Syslog_Component__
#define __H_Syslog_Component__

#ifndef __COMPONENTS__
#include <Components.h>
#endif

#ifndef __PROCESSES__
#include <Processes.h>
#endif

// include the syslog stuff
#include "sys/syslog.h"

#ifndef kComponentWildCard
#define kComponentWildCard 0L
#endif

// Define the type/Manufacturer for the component
#define ksyslogComponentType 'sLog'

// Define the version for the component
#define kVersion 0x00010000

// Define special selectors exclusive to this component
#define kopenlogSelector			0x10
#define ksyslogSelector			0x11
#define kcloselogSelector			0x12
#define ksetlogmaskSelector		0x13
#define kSetFileSelector			0x14

#define kGetErrorSelector		0x00

// For each of the selectors, define the interface
#ifdef __cplusplus
extern "C" {
#endif

// define component specific routines here

pascal ComponentResult Comp_geterror(ComponentInstance inst,OSErr* err) \
	ComponentCallNow(kGetErrorSelector,sizeof(OSErr*));

// open the log message here
pascal ComponentResult Comp_openlog(ComponentInstance inst,const char* ident,long logopt,short facility)\
	ComponentCallNow(kopenlogSelector,sizeof(char*)+sizeof(long)+sizeof(short));

// log a message (note that the normal routine is implemented in glue)
pascal ComponentResult Comp_syslog(ComponentInstance inst,short priority,const char* message)\
	ComponentCallNow(ksyslogSelector,sizeof(short)+sizeof(char*));

// close the opened syslog
pascal ComponentResult Comp_closelog(ComponentInstance inst)\
	ComponentCallNow(kcloselogSelector,0);

// set the syslog mask...
pascal ComponentResult Comp_setlogmask(ComponentInstance inst,short maskpri) \
	ComponentCallNow(ksetlogmaskSelector,sizeof(short));

pascal ComponentResult Comp_setfile(ComponentInstance inst,FSSpecPtr spec)\
	ComponentCallNow(kSetFileSelector,sizeof(FSSpecPtr));

#ifdef __cplusplus
}
#endif

#endif /* __H_Syslog_Component__ */







